@@ -1,18 +1,30 @@
- /*
- 				back.c
-
-*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+/*
+*				back.c
 *
-*	Part of:	SWarp
+* Functions dealing with the image background.
 *
-*	Author:		E.BERTIN (IAP)
+*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 *
-*	Contents:	functions dealing with background computation.
+*	This file part of:	SWarp
 *
-*	Last modify:	25/08/2010
+*	Copyright:		(C) 2000-2010 Emmanuel Bertin -- IAP/CNRS/UPMC
 *
-*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-*/
+*	License:		GNU General Public License
+*
+*	SWarp is free software: you can redistribute it and/or modify
+*	it under the terms of the GNU General Public License as published by
+*	the Free Software Foundation, either version 3 of the License, or
+*	(at your option) any later version.
+*	SWarp is distributed in the hope that it will be useful,
+*	but WITHOUT ANY WARRANTY; without even the implied warranty of
+*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+*	GNU General Public License for more details.
+*	You should have received a copy of the GNU General Public License
+*	along with SWarp. If not, see <http://www.gnu.org/licenses/>.
+*
+*	Last modified:		26/10/2010
+*
+*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
 
 #ifdef HAVE_CONFIG_H
 #include	"config.h"
@@ -33,6 +45,7 @@
 #include	"back.h"
 #include	"coadd.h"
 #include	"field.h"
+#include	"misc.h"
 #include	"prefs.h"
 #include	"weight.h"
 
@@ -71,7 +84,7 @@ void	make_back(fieldstruct *field, fieldstruct *wfield, int wscale_flag)
   ny = field->nbacky;
   nb = field->nback;
 
-  NFPRINTF(OUTPUT, "Setting up background maps");
+  NFPRINTF(OUTPUT, "Setting up background maps ...");
 
 /* Decide if it is worth displaying progress each 16 lines */
 
@@ -150,7 +163,7 @@ void	make_back(fieldstruct *field, fieldstruct *wfield, int wscale_flag)
         }
 /*---- Build the histograms */
       backstat(backmesh, wbackmesh, buf, wbuf, bufsize,nx, w, bw,
-	wfield?wfield->weight_thresh:0.0);
+	wfield?wfield->var_thresh:0.0);
       bm = backmesh;
       for (m=nx; m--; bm++)
         if (bm->mean <= -BIG)
@@ -167,7 +180,7 @@ void	make_back(fieldstruct *field, fieldstruct *wfield, int wscale_flag)
             QCALLOC(wbm->histo, int, wbm->nlevels);
         }
       backhisto(backmesh, wbackmesh, buf, wbuf, bufsize,nx, w, bw,
-	wfield?wfield->weight_thresh:0.0);
+	wfield?wfield->var_thresh:0.0);
       }
     else
       {
@@ -219,7 +232,7 @@ void	make_back(fieldstruct *field, fieldstruct *wfield, int wscale_flag)
           }
         }
       backstat(backmesh, wbackmesh, buf, wbuf, bufsize, nx, w, bw,
-	wfield?wfield->weight_thresh:0.0);
+	wfield?wfield->var_thresh:0.0);
       QFSEEK(tab->cat->file, fcurpos2, SEEK_SET, field->filename);
       bm = backmesh;
       for (m=nx; m--; bm++)
@@ -249,7 +262,7 @@ void	make_back(fieldstruct *field, fieldstruct *wfield, int wscale_flag)
           weight_to_var(wbuf, bufsize2);
           }
         backhisto(backmesh, wbackmesh, buf, wbuf, bufsize2, nx, w, bw,
-		wfield?wfield->weight_thresh:0.0);
+		wfield?wfield->var_thresh:0.0);
         }
       }
 
@@ -288,7 +301,7 @@ void	make_back(fieldstruct *field, fieldstruct *wfield, int wscale_flag)
     QFSEEK(wfield->tab->cat->file, wfcurpos, SEEK_SET, wfield->filename);
 
 /* Median-filter and check suitability of the background map */
-  NFPRINTF(OUTPUT, "Filtering background map(s)");
+  NFPRINTF(OUTPUT, "Filtering background map(s) ...");
   filter_back(field);
   if (wfield)
     filter_back(wfield);
@@ -308,27 +321,27 @@ void	make_back(fieldstruct *field, fieldstruct *wfield, int wscale_flag)
         *(ratiop++) = sratio;
         nr++;
         }
-    wfield->sigfac = (double)hmedian(ratio, nr);
+    wfield->sigfac = (double)fqmedian(ratio, nr);
     for (i=0; i<nr && ratio[i]<=0.0; i++);
     if (i<nr)
-      wfield->sigfac = (double)hmedian(ratio+i, nr-i);
+      wfield->sigfac = (double)fqmedian(ratio+i, nr-i);
     else
       {
       warning("Null or negative global weighting factor:","defaulted to 1");
       wfield->sigfac = 1.0;
       } 
 
-/*-- Update weight threshold */
-    wfield->weight_thresh *= (PIXTYPE)(wfield->sigfac*wfield->sigfac);
+/*-- Update variance threshold */
+    wfield->var_thresh *= (PIXTYPE)(wfield->sigfac*wfield->sigfac);
 
     free(ratio);
     }
 
 /* Compute 2nd derivatives along the y-direction */
-  NFPRINTF(OUTPUT, "Computing backgound d-map");
+  NFPRINTF(OUTPUT, "Computing backgound d-map ...");
   free(field->dback);
   field->dback = make_backspline(field, field->back);
-  NFPRINTF(OUTPUT, "Computing backgound-noise d-map");
+  NFPRINTF(OUTPUT, "Computing backgound-noise d-map ...");
   free(field->dsigma);
   field->dsigma = make_backspline(field, field->sigma);
 /* If asked for, force the backmean parameter to the supplied value */
@@ -734,10 +747,10 @@ void	filter_back(fieldstruct *field)
           smask[i++] = sigma[x+y];
           }
         }
-      if (fabs((med=hmedian(bmask, i))-back[px+py])>=fthresh)
+      if (fabs((med=fqmedian(bmask, i))-back[px+py])>=fthresh)
         {
         back2[px+py] = med;
-        sigma2[px+py] = hmedian(smask, i);
+        sigma2[px+py] = fqmedian(smask, i);
         }
       else
         {
@@ -750,17 +763,17 @@ void	filter_back(fieldstruct *field)
   free(bmask);
   free(smask);
   memcpy(back, back2, np*sizeof(float));
-  field->backmean = (double)hmedian(back2, np);
+  field->backmean = (double)fqmedian(back2, np);
   free(back2);
   memcpy(sigma, sigma2, np*sizeof(float));
-  field->backsig = (double)hmedian(sigma2, np);
+  field->backsig = (double)fqmedian(sigma2, np);
 
   if (field->backsig<=0.0)
     {
     sigmat = sigma2+np;
     for (i=np; i-- && *(--sigmat)>0.0;);
     if (i>=0 && i<(np-1))
-      field->backsig = hmedian(sigmat+1, np-1-i);
+      field->backsig = fqmedian(sigmat+1, np-1-i);
     else
       {
       if (field->flags&(DETECT_FIELD|MEASURE_FIELD))
