@@ -1,18 +1,30 @@
- /*
- 				field.c
-
-*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+/*
+*				field.c
 *
-*	Part of:	SWarp
+* Manage fields (input images).
 *
-*	Author:		E.BERTIN (IAP)
+*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 *
-*	Contents:	Handling of field structures.
+*	This file part of:	SWarp
 *
-*	Last modify:	01/09/2010
+*	Copyright:		(C) 2000-2013 Emmanuel Bertin -- IAP/CNRS/UPMC
 *
-*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-*/
+*	License:		GNU General Public License
+*
+*	SWarp is free software: you can redistribute it and/or modify
+*	it under the terms of the GNU General Public License as published by
+*	the Free Software Foundation, either version 3 of the License, or
+*	(at your option) any later version.
+*	SWarp is distributed in the hope that it will be useful,
+*	but WITHOUT ANY WARRANTY; without even the implied warranty of
+*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+*	GNU General Public License for more details.
+*	You should have received a copy of the GNU General Public License
+*	along with SWarp. If not, see <http://www.gnu.org/licenses/>.
+*
+*	Last modified:		06/06/2013
+*
+*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
 
 #ifdef HAVE_CONFIG_H
 #include	"config.h"
@@ -37,6 +49,7 @@
 #include "field.h"
 #include "header.h"
 #include "key.h"
+#include "misc.h"
 #include "prefs.h"
 #include "wcs/wcs.h"
 
@@ -90,7 +103,7 @@ fieldstruct	*load_field(catstruct *cat, int frameno, int fieldno)
     pstr = field->hfilename+strlen(field->hfilename);
   sprintf(pstr, "%s", prefs.head_suffix);
 
-  sprintf(gstr, "Looking for %s", field->rfilename);
+  sprintf(gstr, "Looking for %s ...", field->rfilename);
   NFPRINTF(OUTPUT, gstr);
 
 /* Insert additional header informations from the "header" file */
@@ -179,7 +192,7 @@ INPUT	Reference field pointer.
 OUTPUT	The new field pointer if OK, NULL otherwise.
 NOTES	-.
 AUTHOR	E. Bertin (IAP)
-VERSION	10/08/2001
+VERSION	24/01/2012
  ***/
 fieldstruct	*inherit_field(char *filename, fieldstruct *reffield,
 				int flags)
@@ -201,12 +214,13 @@ fieldstruct	*inherit_field(char *filename, fieldstruct *reffield,
   field->dback = NULL;
   field->sigma = NULL;
   field->dsigma = NULL;
+  field->ipix = NULL;
   field->pix = NULL;
   field->backline = NULL;
   field->wcs = NULL;
   field->rawmin = NULL;
   field->rawmax = NULL;
-  field->reffield = NULL;
+  field->reffield =reffield;
 
   strcpy(field->filename, filename);
   strcpy(field->cat->filename, filename);
@@ -243,6 +257,7 @@ void	end_field(fieldstruct *field)
 
   end_back(field);
   field->pix = NULL;
+  field->ipix = NULL;
   free(field);
 
   return;
@@ -344,9 +359,9 @@ INPUT	Input field ptr array,
 	number of input fields,
 	Filename.
 OUTPUT	Pointer to the new output field.
-NOTES   -.
+NOTES   Global preferences are used.
 AUTHOR  E. Bertin (IAP)
-VERSION 01/09/2010
+VERSION 06/06/2013
  ***/
 fieldstruct *init_field(fieldstruct **infield, int ninput, char *filename)
   {
@@ -429,7 +444,7 @@ fieldstruct *init_field(fieldstruct **infield, int ninput, char *filename)
   QMALLOC(scale, float, ninput);
   QMALLOC(tab->naxisn, int, naxis);
 /* Produce a floating point output */
-  tab->bitpix = BP_FLOAT;
+  field->bitpix = tab->bitpix = prefs.outfield_bitpix;
 
   if (prefs.resample_flag)
     {
@@ -594,7 +609,7 @@ fieldstruct *init_field(fieldstruct **infield, int ninput, char *filename)
         case PIXSCALE_MEDIAN:
           for (i=0; i<ninput; i++)
             scale[i] = (float)infield[i]->wcs->wcsscale[axis[i*naxis+n]];
-          pixscale[n] = (double)hmedian(scale, ninput);
+          pixscale[n] = (double)fqmedian(scale, ninput);
           break;
         case PIXSCALE_FIT:
           if (prefs.center_type[n] == CENTER_MANUAL)
@@ -665,7 +680,7 @@ fieldstruct *init_field(fieldstruct **infield, int ninput, char *filename)
 		wcsmax[n] - wcs->crval[n];
           if ((n==lng || n==lat) && lat!=lng)
             {
-            if (val<0.0)
+            if (val<-180.0)
               val += 360.0;
             wcs->crpix[n] = (int)(fmod(val, 180.0)/pixscale[n]) + 1.0;
 /*-------- Add a 5% margin in field size */
@@ -863,7 +878,7 @@ fieldstruct *init_field(fieldstruct **infield, int ninput, char *filename)
 	}
       wcs_to_raw(wcs, wcscenter, rawcenter);
       for (n=0; n<naxis; n++)
-        wcs->crpix[n] += (int)(wcs->naxisn[n]/2 - rawcenter[n] +0.49);
+        wcs->crpix[n] += (int)(wcs->naxisn[n]/2 - rawcenter[n] +0.9999);
       }
     }
 
@@ -885,9 +900,9 @@ fieldstruct *init_field(fieldstruct **infield, int ninput, char *filename)
 /* Insert additional header informations from the "header" file */
   if (read_aschead(field->hfilename, 0, tab))
     {
+/*-- No external header: update WCS internal structures only */
     if (!wcs->wcsprm)
       QCALLOC(wcs->wcsprm, struct wcsprm, 1);
-/*--- No external header: update WCS internal structures only */
 /*-- Test if the WCS is recognized and a celestial pair is found */
     wcsset(wcs->naxis,(const char(*)[9])wcs->ctype, wcs->wcsprm);
 
