@@ -1,18 +1,31 @@
 /*
- 				fitswcs.c
-
-*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+*				fitswcs.c
 *
-*	Part of:	LDACTools+
+* Manage World Coordinate System data.
 *
-*	Author:		E.BERTIN (IAP)
+*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 *
-*	Contents:       Read and write WCS header info.
+*	This file part of:	AstrOmatic software
 *
-*	Last modify:	01/09/2010
+*	Copyright:		(C) 1993-2013 Emmanuel Bertin -- IAP/CNRS/UPMC
 *
-*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-*/
+*	License:		GNU General Public License
+*
+*	AstrOmatic software is free software: you can redistribute it and/or
+*	modify it under the terms of the GNU General Public License as
+*	published by the Free Software Foundation, either version 3 of the
+*	License, or (at your option) any later version.
+*	AstrOmatic software is distributed in the hope that it will be useful,
+*	but WITHOUT ANY WARRANTY; without even the implied warranty of
+*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+*	GNU General Public License for more details.
+*	You should have received a copy of the GNU General Public License
+*	along with AstrOmatic software.
+*	If not, see <http://www.gnu.org/licenses/>.
+*
+*	Last modified:		15/11/2013
+*
+*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
 
 #ifdef HAVE_CONFIG_H
 #include	"config.h"
@@ -315,7 +328,7 @@ INPUT	tab structure.
 OUTPUT	-.
 NOTES	-.
 AUTHOR	E. Bertin (IAP)
-VERSION	20/02/2009
+VERSION	15/11/2013
  ***/
 wcsstruct	*read_wcs(tabstruct *tab)
 
@@ -384,7 +397,6 @@ wcsstruct	*read_wcs(tabstruct *tab)
     }
 
   if (fitsfind(buf, "CD?_????")!=RETURN_ERROR)
-    {
 /*-- If CD keywords exist, use them for the linear mapping terms... */
     for (l=0; l<naxis; l++)
       for (j=0; j<naxis; j++)
@@ -392,8 +404,16 @@ wcsstruct	*read_wcs(tabstruct *tab)
         sprintf(str, "CD%d_%d", l+1, j+1);
         FITSREADF(buf, str, wcs->cd[l*naxis+j], l==j?1.0:0.0)
         }
-    }
-  else if (fitsfind(buf, "PC00?00?")!=RETURN_ERROR)
+  else if (fitsfind(buf, "PC?_????")!=RETURN_ERROR)
+/*-- ...If PC keywords exist, use them for the linear mapping terms... */
+    for (l=0; l<naxis; l++)
+      for (j=0; j<naxis; j++)
+        {
+        sprintf(str, "PC%d_%d", l+1, j+1);
+        FITSREADF(buf, str, wcs->cd[l*naxis+j], l==j?1.0:0.0)
+        wcs->cd[l*naxis+j] *= wcs->cdelt[l];
+        }
+  else if (fitsfind(buf, "PC0??0??")!=RETURN_ERROR)
 /*-- ...If PC keywords exist, use them for the linear mapping terms... */
     for (l=0; l<naxis; l++)
       for (j=0; j<naxis; j++)
@@ -426,6 +446,8 @@ wcsstruct	*read_wcs(tabstruct *tab)
 /*-- Coordinate reference frame */
 /*-- Search for an observation date expressed in Julian days */
     FITSREADF(buf, "MJD-OBS ", date, -1.0);
+    if (date<0.0)
+      FITSREADF(buf, "MJDSTART", date, -1.0);
 /*-- Precession date (defined from Ephemerides du Bureau des Longitudes) */
 /*-- in Julian years from 2000.0 */
     if (date>0.0)
@@ -433,7 +455,7 @@ wcsstruct	*read_wcs(tabstruct *tab)
     else
       {
 /*---- Search for an observation date expressed in "civilian" format */
-      FITSREADS(buf, "DATE-OBS ", str, "");
+      FITSREADS(buf, "DATE-OBS", str, "");
       if (*str)
         {
 /*------ Decode DATE-OBS format: DD/MM/YY or YYYY-MM-DD */
@@ -466,7 +488,8 @@ wcsstruct	*read_wcs(tabstruct *tab)
 
     FITSREADF(buf, "EPOCH", wcs->epoch, 2000.0);
     FITSREADF(buf, "EQUINOX", wcs->equinox, wcs->epoch);
-    FITSREADS(buf, "RADECSYS", str,
+    if (fitsread(buf, "RADESYS", str, H_STRING,T_STRING) != RETURN_OK)
+      FITSREADS(buf, "RADECSYS", str,
 	wcs->equinox >= 2000.0? "ICRS" : (wcs->equinox<1984.0? "FK4" : "FK5"));
     if (!strcmp(str, "ICRS"))
       wcs->radecsys = RDSYS_ICRS;
@@ -556,7 +579,8 @@ wcsstruct	*read_wcs(tabstruct *tab)
       }
     else
       {
-      FITSREADF(buf, "LONGPOLE", wcs->longpole, 999.0);
+      if (fitsread(buf, "LONPOLE",&wcs->longpole,H_FLOAT,T_DOUBLE) != RETURN_OK)
+        FITSREADF(buf, "LONGPOLE", wcs->longpole, 999.0);
       FITSREADF(buf, "LATPOLE ", wcs->latpole, 999.0);
 /*---- Old convention */
       if (fitsfind(buf, "PROJP???") != RETURN_ERROR)
@@ -600,7 +624,7 @@ INPUT	tab structure,
 OUTPUT	-.
 NOTES	-.
 AUTHOR	E. Bertin (IAP)
-VERSION	01/09/2010
+VERSION	27/11/2013
  ***/
 void	write_wcs(tabstruct *tab, wcsstruct *wcs)
 
@@ -628,26 +652,26 @@ void	write_wcs(tabstruct *tab, wcsstruct *wcs)
     addkeywordto_head(tab, "MJD-OBS ", "Modified Julian date at start");
     fitswrite(tab->headbuf, "MJD-OBS ", &mjd, H_EXPO,T_DOUBLE);
     }
-  addkeywordto_head(tab, "RADECSYS", "Astrometric system");
+  addkeywordto_head(tab, "RADESYS ", "Astrometric system");
   switch(wcs->radecsys)
     {
     case RDSYS_ICRS:
-      fitswrite(tab->headbuf, "RADECSYS", "ICRS", H_STRING, T_STRING);
+      fitswrite(tab->headbuf, "RADESYS ", "ICRS", H_STRING, T_STRING);
       break;
     case RDSYS_FK5:
-      fitswrite(tab->headbuf, "RADECSYS", "FK5", H_STRING, T_STRING);
+      fitswrite(tab->headbuf, "RADESYS ", "FK5", H_STRING, T_STRING);
       break;
     case RDSYS_FK4:
-      fitswrite(tab->headbuf, "RADECSYS", "FK4", H_STRING, T_STRING);
+      fitswrite(tab->headbuf, "RADESYS ", "FK4", H_STRING, T_STRING);
       break;
     case RDSYS_FK4_NO_E:
-      fitswrite(tab->headbuf, "RADECSYS", "FK4-NO-E", H_STRING, T_STRING);
+      fitswrite(tab->headbuf, "RADESYS ", "FK4-NO-E", H_STRING, T_STRING);
       break;
     case RDSYS_GAPPT:
-      fitswrite(tab->headbuf, "RADECSYS", "GAPPT", H_STRING, T_STRING);
+      fitswrite(tab->headbuf, "RADESYS ", "GAPPT", H_STRING, T_STRING);
       break;
     default:
-      error(EXIT_FAILURE, "*Error*: unknown RADECSYS type in write_wcs()", "");
+      error(EXIT_FAILURE, "*Error*: unknown RADESYS type in write_wcs()", "");
     }
   for (l=0; l<naxis; l++)
     {
@@ -670,7 +694,7 @@ void	write_wcs(tabstruct *tab, wcsstruct *wcs)
       fitswrite(tab->headbuf, str, &wcs->cd[l*naxis+j], H_EXPO, T_DOUBLE);
       }
     for (j=0; j<100; j++)
-      if (wcs->projp[j+100*l] != 0.0)
+      if (fabs(wcs->projp[j+100*l]) > TINY)
         {
         sprintf(str, "PV%d_%d", l+1, j);
         addkeywordto_head(tab, str, "Projection distortion parameter");
@@ -730,18 +754,19 @@ INPUT	Proposed projection code name.
 OUTPUT	RETURN_OK if projection is supported, RETURN_ERROR otherwise.
 NOTES	-.
 AUTHOR	E. Bertin (IAP)
-VERSION	24/05/2000
+VERSION	14/06/2012
  ***/
 int	wcs_supproj(char *name)
 
   {
-   char	projcode[26][5] =
-	{"AZP", "TAN", "SIN", "STG", "ARC", "ZPN", "ZEA", "AIR", "CYP", "CAR",
-	"MER", "CEA", "COP", "COD", "COE", "COO", "BON", "PCO", "GLS", "PAR",
-	"AIT", "MOL", "CSC", "QSC", "TSC", "NONE"};
+   char	projcode[28][5] =
+	{"TAN", "TPV", "AZP", "SIN", "STG", "ARC", "ZPN", "ZEA", "AIR", "CYP",
+	"CAR", "MER", "CEA", "COP", "COD", "COE", "COO", "BON", "PCO", "GLS",
+	"PAR", "AIT", "MOL", "CSC", "QSC", "TSC", "TNX", "NONE"};
+
    int	i;
 
-  for (i=0; i<26; i++)
+  for (i=0; i<28; i++)
     if (!strcmp(name, projcode[i]))
       return RETURN_OK;
 
@@ -756,7 +781,7 @@ INPUT	WCS structure.
 OUTPUT	-.
 NOTES	.
 AUTHOR	E. Bertin (IAP)
-VERSION	06/11/2003
+VERSION	13/07/2012
  ***/
 void	invert_wcs(wcsstruct *wcs)
 
@@ -775,7 +800,8 @@ void	invert_wcs(wcsstruct *wcs)
   lat = wcs->wcsprm->lat;
   if (!strcmp(wcs->wcsprm->pcode, "TNX"))
     tnxflag = 1;
-  else if (!strcmp(wcs->wcsprm->pcode, "TAN")
+  else if ((!strcmp(wcs->wcsprm->pcode, "TAN")
+	|| !strcmp(wcs->wcsprm->pcode, "TPV"))
 		&& (wcs->projp[1+lng*100] || wcs->projp[1+lat*100]))
     tnxflag = 0;
   else
@@ -1036,23 +1062,23 @@ void	range_wcs(wcsstruct *wcs)
 
 
 /******* frame_wcs ***********************************************************
-PROTO	void frame_wcs(wcsstruct *wcsin, wcsstruct *wcsout)
+PROTO	int frame_wcs(wcsstruct *wcsin, wcsstruct *wcsout)
 PURPOSE	Find the x and y limits of an input frame in an output image.
 INPUT	WCS structure of the input frame,
 	WCS structure of the output frame.
-OUTPUT	-.
-NOTES	.
+OUTPUT	1 if frames overlap, 0 otherwise.
+NOTES	-.
 AUTHOR	E. Bertin (IAP)
-VERSION	29/12/2004
+VERSION	26/03/2012
  ***/
-void	frame_wcs(wcsstruct *wcsin, wcsstruct *wcsout)
+int	frame_wcs(wcsstruct *wcsin, wcsstruct *wcsout)
 
   {
    double		rawin[NAXIS], rawout[NAXIS], world[NAXIS];
    int			linecount[NAXIS];
    double		worldc;
    int			*min, *max,
-			i,j, naxis, npoints, out, swapflag;
+			i,j, naxis, npoints, out, swapflag, overlapflag;
 
   naxis = wcsin->naxis;
 
@@ -1105,7 +1131,7 @@ void	frame_wcs(wcsstruct *wcsin, wcsstruct *wcsout)
       }
     }
 
-/* Just add a little margin, in case of... */
+/* Add a little margin, just in case... */
   for (i=0; i<naxis; i++)
     {
     if (min[i]>-2147483647)
@@ -1114,7 +1140,16 @@ void	frame_wcs(wcsstruct *wcsin, wcsstruct *wcsout)
       max[i] += 2;
     }
 
-  return;
+/* Check overlap */
+  overlapflag = 1;
+  for (i=0; i<naxis; i++)
+    if (min[i]>wcsout->naxisn[i] || max[i]<0)
+      {
+      overlapflag = 0;
+      break;
+      }
+
+  return overlapflag;
   }
 
 
@@ -1556,7 +1591,7 @@ double	wcs_scale(wcsstruct *wcs, double *pixpos)
 
 /****** wcs jacobian *********************************************************
 PROTO	double wcs_jacobian(wcsstruct *wcs, double *pixpos, double *jacob)
-PURPOSE	Compute the local Jacobian matrice of the astrometric deprojection.
+PURPOSE	Compute the local Jacobian matrix of the astrometric deprojection.
 INPUT	WCS structure,
 	Pointer to the array of local raw coordinates,
 	Pointer to the jacobian array (output).
@@ -1612,6 +1647,70 @@ double	wcs_jacobian(wcsstruct *wcs, double *pixpos, double *jacob)
   }
 
 
+/****** wcs rawtoraw *********************************************************
+PROTO	double wcs_rawtoraw(wcsstruct *wcsin, wcsstruct *wcsout,
+		double *pixpos, double *jacob)
+PURPOSE	Convert raw coordinates from input wcs structure to raw coordinates
+	from output wcs structure, and the local Jacobian matrix of the
+	reprojection.
+INPUT	WCS input structure,
+	WCS output structure,
+	pointer to the array of local input raw coordinates,
+	pointer to the array of local output raw coordinates (output),
+	pointer to the jacobian array (output).
+OUTPUT	Determinant over spatial coordinates (ratio of pixel areas),
+	0.0 if jacob is NULL (Jacobian not computed in that case),
+	or -1.0 if mapping was unsuccesful.
+NOTES   Memory must have been allocated (naxis*naxis*sizeof(double)) for the
+        Jacobian array.
+AUTHOR	E. Bertin (IAP)
+VERSION	12/06/2012
+ ***/
+double	wcs_rawtoraw(wcsstruct *wcsin, wcsstruct *wcsout,
+		double *pixposin, double *pixposout, double *jacob)
+  {
+   double	pixpos0[NAXIS], pixpos2[NAXIS], wcspos[NAXIS];
+   int		i,j, lng,lat,naxis;
+
+  naxis = wcsin->naxis;
+  for (i=0; i<naxis; i++)
+    pixpos0[i] = pixposin[i];
+
+/* Coordinate transformation */
+  if (raw_to_wcs(wcsin, pixposin, wcspos) == RETURN_ERROR)
+    return -1.0;
+  if (wcs_to_raw(wcsout, wcspos, pixposout) == RETURN_ERROR)
+    return -1.0;
+ 
+/* Jacobian */
+  if (!jacob)
+    return 0.0;
+
+  lng = wcsin->lng;
+  lat = wcsin->lat;
+  for (i=0; i<naxis; i++)
+    {
+    pixpos0[i] += 1.0;
+    if (raw_to_wcs(wcsin, pixpos0, wcspos) == RETURN_ERROR)
+      return -1.0;
+    if (wcs_to_raw(wcsout, wcspos, pixpos2) == RETURN_ERROR)
+      return -1.0;
+    pixpos0[i] -= 1.0;
+    for (j=0; j<naxis; j++)
+      jacob[j*naxis+i] = pixpos2[j] - pixposout[j];
+    }
+
+  if (lng==lat)
+    {
+    lng = 0;
+    lat = 1;
+    }
+
+  return fabs(jacob[lng+naxis*lng]*jacob[lat+naxis*lat]
+		- jacob[lat+naxis*lng]*jacob[lng+naxis*lat]);
+  }
+
+
 /******* wcs_chirality *******************************************************
 PROTO	int wcs_chirality(wcsstruct *wcs)
 PURPOSE	Compute the chirality of a WCS projection.
