@@ -1,29 +1,48 @@
 /*
- 				prefs.c
-
-*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+*				prefs.c
 *
-*	Part of:	SWarp
+* Run-time configuration.
 *
-*	Author:		E.BERTIN (IAP)
+*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 *
-*	Contents:	Functions to handle the configuration file.
+*	This file part of:	SWarp
 *
-*	Last modify:	24/08/2010
+*	Copyright:		(C) 2000-2013 Emmanuel Bertin -- IAP/CNRS/UPMC
 *
-*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-*/
+*	License:		GNU General Public License
+*
+*	SWarp is free software: you can redistribute it and/or modify
+*	it under the terms of the GNU General Public License as published by
+*	the Free Software Foundation, either version 3 of the License, or
+*	(at your option) any later version.
+*	SWarp is distributed in the hope that it will be useful,
+*	but WITHOUT ANY WARRANTY; without even the implied warranty of
+*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+*	GNU General Public License for more details.
+*	You should have received a copy of the GNU General Public License
+*	along with SWarp. If not, see <http://www.gnu.org/licenses/>.
+*
+*	Last modified:		22/03/2013
+*
+*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
 
 #ifdef HAVE_CONFIG_H
 #include	"config.h"
 #endif
 
 #include	<ctype.h>
+
 #ifdef HAVE_MATHIMF_H
 #include <mathimf.h>
 #else
 #include <math.h>
 #endif
+
+#ifdef HAVE_GETRLIMIT
+#include 	<sys/time.h>
+#include 	<sys/resource.h>
+#endif
+
 #include	<stdio.h>
 #include	<stdlib.h>
 #include	<string.h>
@@ -70,10 +89,10 @@ documentation)
 void    readprefs(char *filename, char **argkey, char **argval, int narg)
 
   {
-   FILE          *infile;
-   char          *cp, str[MAXCHARL], *keyword, *value, **dp;
-   int           i, ival, nkey, warn, argi, flagc, flagd, flage, flagz;
-   float         dval;
+   FILE		*infile;
+   char		*cp, str[MAXCHARL], *keyword, *value, **dp;
+   int		i, ival, nkey, warn, argi, flagc, flagd, flage, flagz;
+   double	dval;
 #ifdef	HAVE_GETENV
    static char	value2[MAXCHARL],envname[MAXCHAR];
    char		*dolpos, *listbuf;
@@ -512,6 +531,54 @@ void	useprefs(void)
     }
 #endif
 
+#ifdef HAVE_GETRLIMIT
+/* Check that the maximum number of open files is < what the system allows */
+  if (prefs.combine_flag)
+    {
+     struct rlimit	rlim;
+    if (prefs.nopenfiles_max && getrlimit(RLIMIT_NOFILE, &rlim) != -1
+	&& rlim.rlim_cur< RLIM_INFINITY
+	&& prefs.nopenfiles_max > (unsigned int)rlim.rlim_cur)
+      {
+      sprintf(gstr, "%d", (unsigned int)rlim.rlim_cur);
+      warning("NOPENFILES_MAX larger than the system limit currently set at ",
+		gstr);
+      }
+    }
+#endif
+
+/* Trigger integer mode if RESAMPLE is Y and RESAMPLING_TYPE is set to FLAGS */
+  prefs.outfield_bitpix = BP_FLOAT;
+  for (i=0; i<prefs.nresamp_type; i++)
+    if ((prefs.resample_flag) && prefs.resamp_type[i] == INTERP_FLAGS)
+      {
+      prefs.outfield_bitpix = BP_LONG;
+      for (j=0; j<prefs.nresamp_type; j++)
+        prefs.resamp_type[j] = INTERP_FLAGS;
+      break;
+      }
+
+/* Trigger integer mode if COMBINE is Y and boolean arithmetics are selected */
+  if (prefs.combine_flag)
+    {
+    if (prefs.coadd_type == COADD_AND
+	|| prefs.coadd_type == COADD_NAND
+	|| prefs.coadd_type == COADD_OR
+	|| prefs.coadd_type == COADD_NOR)
+      {
+      prefs.outfield_bitpix = BP_LONG;
+      for (j=0; j<prefs.nresamp_type; j++)
+        prefs.resamp_type[j] = INTERP_FLAGS;
+      }
+    else if (prefs.outfield_bitpix == BP_LONG)
+      {
+      for (j=0; j<prefs.nresamp_type; j++)
+        prefs.coadd_type = COADD_OR;
+      warning("COMBINE_TYPE incompatible with RESAMPLING_TYPE FLAGS: ",
+	"Forcing to OR");
+      }
+    }   
+
 /* Force weight flag */
   weight_flag = 0;
   for (i=0; i<prefs.nweight_type; i++)
@@ -632,8 +699,8 @@ void	useprefs(void)
     prefs.nweight_thresh = 1;
     }
   else
-  for (i=prefs.nweight_thresh; i<prefs.ninfield; i++)
-    prefs.weight_thresh[i] = prefs.back_default[prefs.nweight_thresh-1];
+    for (i=prefs.nweight_thresh; i<prefs.ninfield; i++)
+      prefs.weight_thresh[i] = prefs.weight_thresh[prefs.nweight_thresh-1];
   prefs.nweight_thresh = prefs.ninfield;
 
 /* Projection approximation error */
