@@ -1,18 +1,30 @@
 /*
-                                  resample.c
-
-*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+*				resample.c
 *
-*       Part of:        Swarp
+* Manage high-level resampling.
 *
-*       Author:         E.BERTIN (IAP)
+*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 *
-*       Contents:       Resampling procedures
+*	This file part of:	SWarp
 *
-*       Last modify:    25/08/2010
+*	Copyright:		(C) 2000-2013 Emmanuel Bertin -- IAP/CNRS/UPMC
 *
-*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-*/
+*	License:		GNU General Public License
+*
+*	SWarp is free software: you can redistribute it and/or modify
+*	it under the terms of the GNU General Public License as published by
+*	the Free Software Foundation, either version 3 of the License, or
+*	(at your option) any later version.
+*	SWarp is distributed in the hope that it will be useful,
+*	but WITHOUT ANY WARRANTY; without even the implied warranty of
+*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+*	GNU General Public License for more details.
+*	You should have received a copy of the GNU General Public License
+*	along with SWarp. If not, see <http://www.gnu.org/licenses/>.
+*
+*	Last modified:		19/07/2013
+*
+*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
 
 #ifdef HAVE_CONFIG_H
 #include	"config.h"
@@ -66,9 +78,10 @@
 			**rawbufarea,
 			ascale;
  PIXTYPE		**outbuf,**outwbuf;
+ FLAGTYPE		**outibuf,**outwibuf, **oversampibuf,**oversampwibuf;
  int			**oversampnbuf, *oversamp,
 			noversamp, oversampflag, width, height, naxis, nlines,
-			approxflag, dispstep;
+			approxflag, dispstep, iflag;
  char			padbuf[FBSIZE];
 
 /*------------------------------ function -----------------------------------*/
@@ -94,7 +107,7 @@ OUTPUT	-.
 NOTES	The structure pointers pointed by pinfield and and pinwfield are
 	updated and point to the resampled fields on output.
 AUTHOR	E. Bertin (IAP)
-VERSION	25/08/2010
+VERSION	19/07/2013
  ***/
 void	resample_field(fieldstruct **pinfield, fieldstruct **pinwfield,
 		fieldstruct *outfield, fieldstruct *outwfield,
@@ -142,14 +155,18 @@ void	resample_field(fieldstruct **pinfield, fieldstruct **pinwfield,
   field->fsaturation = infield->fsaturation;
   field->exptime = infield->exptime;
   field->fieldno = infield->fieldno;
+  field->fascale = infield->fascale;
   field->fscale = infield->fscale;
   field->headflag = infield->headflag;
   strcpy(field->ident, infield->ident);
+  iflag = (outfield->bitpix>0);
 
 /* Now modify some characteristics of the output file */
 /* We use a small "dirty" trick to frame the output */
   wcsloc = *(outfield->wcs);	/* Copy the CONTENT only */
   wcs = infield->wcs;
+  wcsloc.obsdate = wcs->obsdate;
+  wcsloc.epoch = wcs->epoch;
   naxis = wcsloc.naxis;
   for (d=0; d<naxis;d++)
     {
@@ -214,6 +231,11 @@ void	resample_field(fieldstruct **pinfield, fieldstruct **pinwfield,
 /* Add relevant information to FITS headers */
   writefitsinfo_field(wfield, inwfield? inwfield : infield);
   wfield->cflags = 0;
+  if (inwfield)
+    {
+    wfield->sigfac = inwfield->sigfac;
+    wfield->weight_thresh = inwfield->weight_thresh;
+    }
 
 /* Write image header */
   if (open_cat(wfield->cat, WRITE_ONLY) != RETURN_OK)
@@ -245,7 +267,7 @@ void	resample_field(fieldstruct **pinfield, fieldstruct **pinwfield,
 /* Turn approximation on or off */
   approxflag = ((projerr = prefs.proj_err[infield->fieldno] > 0.0)
     && (projapp = projapp_init(infield->wcs, field->wcs, projerr,
-	prefs.fscalastro_type==FSCALASTRO_VARIABLE, infield->fascale)));
+	prefs.fscalastro_type==FSCALASTRO_VARIABLE)));
 
 #ifdef USE_THREADS
 /* Set up multi-threading stuff */
@@ -268,14 +290,23 @@ void	resample_field(fieldstruct **pinfield, fieldstruct **pinwfield,
 /* Initialize the astrometric vector */
   for (d=0; d<naxis;d++)
     {
-    rawmin[d] = rawpos0[d] = 0.5 + 0.5*(stepover[d]=1.0/oversamp[d]);
+    stepover[d]=1.0/oversamp[d];
+    rawmin[d] = rawpos0[d] = 0.5 + 0.5*stepover[d];
     rawmax[d] = (double)wcs->naxisn[d];
     }
 
 /*-- Allocate memory for buffer pointers */
   QMALLOC(rawposp, double *, nlines);
-  QMALLOC(outbuf, PIXTYPE *, nlines);
-  QMALLOC(outwbuf, PIXTYPE *, nlines);
+  if (iflag)
+    {
+    QMALLOC(outibuf, FLAGTYPE *, nlines);
+    QMALLOC(outwibuf, FLAGTYPE *, nlines);
+    }
+  else
+    {
+    QMALLOC(outbuf, PIXTYPE *, nlines)
+    QMALLOC(outwbuf, PIXTYPE *, nlines);
+    }
   QMALLOC(rawbuf, double *, nlines);
   QCALLOC(rawbufarea, double *, nlines);
   QMALLOC(ikernel, ikernelstruct *, nlines);
@@ -283,8 +314,16 @@ void	resample_field(fieldstruct **pinfield, fieldstruct **pinwfield,
   QMALLOC(wcsoutp, wcsstruct *, nlines);
   if (oversampflag)
     {
-    QMALLOC(oversampbuf, double *, nlines);
-    QMALLOC(oversampwbuf, double *, nlines);
+    if (iflag)
+      {
+      QMALLOC(oversampibuf, FLAGTYPE *, nlines)
+      QMALLOC(oversampwibuf, FLAGTYPE *, nlines);
+      }
+    else
+      {
+      QMALLOC(oversampbuf, double *, nlines)
+      QMALLOC(oversampwbuf, double *, nlines);
+      }
     QMALLOC(oversampnbuf, int *, nlines);
     }
 
@@ -292,19 +331,35 @@ void	resample_field(fieldstruct **pinfield, fieldstruct **pinwfield,
     {
 /*-- Allocate memory for the output buffers that contain the final data in */
 /*-- internal format (PIXTYPE) */
-    QMALLOC(outbuf[l], PIXTYPE, width);
-    QMALLOC(outwbuf[l], PIXTYPE, width);
+    if (iflag)
+      {
+      QMALLOC(outibuf[l], FLAGTYPE, width)
+      QMALLOC(outwibuf[l], FLAGTYPE, width);
+      }
+    else
+      {
+      QMALLOC(outbuf[l], PIXTYPE, width)
+      QMALLOC(outwbuf[l], PIXTYPE, width);
+      }
     if (oversampflag)
 /*---- Provide memory space for integrating the content of oversampled pixels*/
       {
-      QMALLOC(oversampbuf[l], double, width);
-      QMALLOC(oversampwbuf[l], double, width);
+      if (iflag)
+        {
+        QMALLOC(oversampibuf[l], FLAGTYPE, width)
+        QMALLOC(oversampwibuf[l], FLAGTYPE, width);
+        }
+      else
+        {
+        QMALLOC(oversampbuf[l], double, width)
+        QMALLOC(oversampwbuf[l], double, width);
+        }
       QMALLOC(oversampnbuf[l], int, width);
       }
 /*-- Provide memory space for the current astrometric line */
     QMALLOC(rawbuf[l], double, naxis*width);
     if (prefs.fscalastro_type==FSCALASTRO_VARIABLE)
-      QMALLOC(rawbufarea[l], double, width);
+      QMALLOC(rawbufarea[l], double, width)
     QMALLOC(rawposp[l], double, naxis);
 /*-- Initialize interpolation kernel */
     ikernel[l] = init_ikernel(interptype, naxis);
@@ -348,8 +403,16 @@ void	resample_field(fieldstruct **pinfield, fieldstruct **pinwfield,
       NPRINTF(OUTPUT, "\33[1M> Resampling line:%7d / %-7d\n\33[1A",
 	y, height);
     warp_line(0);
-    write_body(field->tab, outbuf[0], width);
-    write_body(wfield->tab, outwbuf[0], width);
+    if (iflag)
+      {
+      write_ibody(field->tab, outibuf[0], width);
+      write_ibody(wfield->tab, outwibuf[0], width);
+      }
+    else
+      {
+      write_body(field->tab, outbuf[0], width);
+      write_body(wfield->tab, outwbuf[0], width);
+      }
     }
 #endif
 
@@ -389,12 +452,12 @@ void	resample_field(fieldstruct **pinfield, fieldstruct **pinwfield,
   for (l=0; l<nlines; l++)
     {
     free(rawposp[l]);
-    free(outbuf[l]);
-    free(outwbuf[l]);
+    free(iflag? (void *)outibuf[l] : (void *)outbuf[l]);
+    free(iflag? (void *)outwibuf[l]: (void *)outwbuf[l]);
     if (oversampflag)
       {
-      free(oversampbuf[l]);
-      free(oversampwbuf[l]);
+      free(iflag? (void *)oversampibuf[l] : (void *)oversampbuf[l]);
+      free(iflag? (void *)oversampwibuf[l]: (void *)oversampwbuf[l]);
       free(oversampnbuf[l]);
       }
     free(rawbuf[l]);
@@ -405,12 +468,12 @@ void	resample_field(fieldstruct **pinfield, fieldstruct **pinwfield,
     end_wcs(wcsoutp[l]);
     }
   free(rawposp);
-  free(outbuf);
-  free(outwbuf);
+  free(iflag? (void *)outibuf : (void *)outbuf);
+  free(iflag? (void *)outwibuf: (void *)outwbuf);
   if (oversampflag)
     {
-    free(oversampbuf);
-    free(oversampwbuf);
+    free(iflag? (void *)oversampibuf : (void *)oversampbuf);
+    free(iflag? (void *)oversampwibuf: (void *)oversampwbuf);
     free(oversampnbuf);
     }
   free(rawbuf);
@@ -462,7 +525,7 @@ INPUT	-.
 OUTPUT	Next available line index.
 NOTES	-.
 AUTHOR	E. Bertin (IAP)
-VERSION	15/09/2005
+VERSION	03/02/2012
  ***/
 int	pthread_nextline(int l)
   {
@@ -478,8 +541,16 @@ int	pthread_nextline(int l)
     {
     while (writeflag[writeline]==2)
       {
-      write_body(ftab, outbuf[writeline], width);
-      write_body(wtab, outwbuf[writeline], width);
+      if (iflag)
+        {
+        write_ibody(ftab, outibuf[writeline], width);
+        write_ibody(wtab, outwibuf[writeline], width);
+        }
+      else
+        {
+        write_body(ftab, outbuf[writeline], width);
+        write_body(wtab, outwbuf[writeline], width);
+        }
       writeflag[writeline] = 0;
       QPTHREAD_COND_BROADCAST(&linecond[writeline]);
       writeline = (writeline+1)%nlines;
@@ -548,7 +619,7 @@ INPUT	Thread number.
 OUTPUT	-.
 NOTES	-.
 AUTHOR	E. Bertin (IAP)
-VERSION	25/08/2010
+VERSION	05/02/2012
  ***/
 void	warp_line(int p)
   {
@@ -558,12 +629,22 @@ void	warp_line(int p)
 			area, worldc;
    PIXTYPE		*out, *outw,
 			pix,pixw;
+   FLAGTYPE		*outi,*outwi, *oversampit,*oversampwit,
+			ipix,ipixw;
    int			nstepover[NAXIS],stepcount[NAXIS],
 			*oversampnt,
 			d, o, x, ninput, swapflag;
 
-  out = outbuf[p];
-  outw = outwbuf[p];
+  if (iflag)
+    {
+    outi = outibuf[p];
+    outwi = outwibuf[p];
+    }
+  else
+    {
+    out = outbuf[p];
+    outw = outwbuf[p];
+    }
   rawpos = rawposp[p];
   wcsin = wcsinp[p];
   wcsout = wcsoutp[p];
@@ -582,8 +663,16 @@ void	warp_line(int p)
       stepcount[d] = nstepover[d] = 0;
       rawposover[d] = rawpos[d];
       }
-    memset(oversampbuf[p], 0, sizeof(double)*width);
-    memset(oversampwbuf[p], 0, sizeof(double)*width);
+    if (iflag)
+      {
+      memset(oversampibuf[p], 0, sizeof(FLAGTYPE)*width);
+      memset(oversampwibuf[p], 0, sizeof(FLAGTYPE)*width);
+      }
+    else
+      {
+      memset(oversampbuf[p], 0, sizeof(double)*width);
+      memset(oversampwbuf[p], 0, sizeof(double)*width);
+      }
     memset(oversampnbuf[p], 0, sizeof(int)*width);
     for (o=noversamp; o--; )
       {
@@ -618,28 +707,58 @@ void	warp_line(int p)
           }
 	}
 /*---- Resample the current line */
-      oversampt = oversampbuf[p];
-      oversampwt = oversampwbuf[p];
-      oversampnt = oversampnbuf[p];
-      rawbufc = rawbuf[p];
-      rawbufareac = rawbufarea[p];
-      for (x=width; x--; rawbufc+=naxis)
+      if (iflag)
         {
-        if (rawbufareac)
-          area = *(rawbufareac++);
-        if (*rawbufc != WCS_NOCOORD
+        oversampit = oversampibuf[p];
+        oversampwit = oversampwibuf[p];
+        oversampnt = oversampnbuf[p];
+        rawbufc = rawbuf[p];
+        rawbufareac = rawbufarea[p];
+        for (x=width; x--; rawbufc+=naxis)
+          {
+          if (rawbufareac)
+            area = *(rawbufareac++);
+          if (*rawbufc != WCS_NOCOORD
+		&& interpolate_ipix(infield, inwfield, rawbufc,&ipix,&ipixw)
+			== RETURN_OK)
+            {
+            *(oversampit++) |= ipix;
+            *(oversampwit++) |= 1;
+            (*(oversampnt++))++;
+            }
+          else
+            {
+            oversampit++;
+            oversampwit++;
+            oversampnt++;
+            }
+          }
+        }
+      else
+        {
+        oversampt = oversampbuf[p];
+        oversampwt = oversampwbuf[p];
+        oversampnt = oversampnbuf[p];
+        rawbufc = rawbuf[p];
+        rawbufareac = rawbufarea[p];
+        for (x=width; x--; rawbufc+=naxis)
+          {
+          if (rawbufareac)
+            area = *(rawbufareac++);
+          if (*rawbufc != WCS_NOCOORD
 		&& (interpolate_pix(infield, inwfield, ikernel[p],rawbufc,
 			&pix,&pixw),pixw<BIG))
-          {
-          *(oversampt++) += area * (double)pix;
-          *(oversampwt++) += (double)pixw * area*area;
-          (*(oversampnt++))++;
-          }
-        else
-          {
-          oversampt++;
-          oversampwt++;
-          oversampnt++;
+            {
+            *(oversampt++) += area * (double)pix;
+            *(oversampwt++) += (double)pixw * area*area;
+            (*(oversampnt++))++;
+            }
+          else
+            {
+            oversampt++;
+            oversampwt++;
+            oversampnt++;
+            }
           }
         }
 /*---- Update coordinate vector */
@@ -656,19 +775,39 @@ void	warp_line(int p)
         }
       }
 /*-- Now transfer to the output line */
-    oversampt = oversampbuf[p];
-    oversampwt = oversampwbuf[p];
-    oversampnt = oversampnbuf[p];
-    for (x=width; x--; oversampt++, oversampwt++)
+    if (iflag)
       {
-      if ((ninput = *(oversampnt++)))
+      oversampit = oversampibuf[p];
+      oversampwit = oversampwibuf[p];
+      oversampnt = oversampnbuf[p];
+      for (x=width; x--; oversampit++, oversampwit++)
         {
-        *(out++) = (PIXTYPE)(*oversampt/ninput);
-/*------- Convert variances to weight */
-        *(outw++) = (PIXTYPE)((ninput / *oversampwt)*ascale);
+        if ((ninput = *(oversampnt++)))
+          {
+          *(outi++) = *oversampit;
+/*--------- Convert variances to weight */
+          *(outwi++) = *oversampwit;
+          }
+        else
+          *(outwi++) = *(outi++) = 0;
+        }
+      }
+    else
+      {
+      oversampt = oversampbuf[p];
+      oversampwt = oversampwbuf[p];
+      oversampnt = oversampnbuf[p];
+      for (x=width; x--; oversampt++, oversampwt++)
+        {
+        if ((ninput = *(oversampnt++)))
+          {
+          *(out++) = (PIXTYPE)(*oversampt/ninput);
+/*--------- Convert variances to weight */
+          *(outw++) = (PIXTYPE)((ninput / *oversampwt)*ascale);
+          }
+        else
+          *(out++) = *(outw++) = 0.0;
         }
-      else
-        *(out++) = *(outw++) = 0.0;
       }
     }
   else
@@ -705,21 +844,32 @@ void	warp_line(int p)
 /*-- Resample the line */
     rawbufc = rawbuf[p];
     rawbufareac = rawbufarea[p];
-    for (x=width; x--; rawbufc+=naxis)
-      {
-      if (rawbufareac)
-        area = *(rawbufareac++);
-      if (*rawbufc != WCS_NOCOORD)
+    if (iflag)
+      for (x=width; x--; rawbufc+=naxis)
         {
-        interpolate_pix(infield, inwfield, ikernel[p], rawbufc,out,outw);
-        *(out++) *= area;
-/*----- Convert variance to weight */
-        *outw = (*outw < BIG) ? 1.0/(*outw*area*area) : 0.0;
-        outw++;
+        if (rawbufareac)
+          area = *(rawbufareac++);
+        if (*rawbufc != WCS_NOCOORD)
+          interpolate_ipix(infield, inwfield, rawbufc,outi++,outwi++);
+        else
+          *(outwi++) = *(outi++) = 0;
+        }
+    else
+      for (x=width; x--; rawbufc+=naxis)
+        {
+        if (rawbufareac)
+          area = *(rawbufareac++);
+        if (*rawbufc != WCS_NOCOORD)
+          {
+          interpolate_pix(infield, inwfield, ikernel[p], rawbufc,out,outw);
+          *(out++) *= area;
+/*------- Convert variance to weight */
+          *outw = (*outw < BIG) ? 1.0/(*outw*area*area) : 0.0;
+          outw++;
+          }
+        else
+          *(out++) = *(outw++) = 0.0;
         }
-      else
-        *(out++) = *(outw++) = 0.0;
-      }
     }
 
   return;
