@@ -1,18 +1,30 @@
- /*
-				xml.c
-
-*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+/*
+*				xml.c
 *
-*	Part of:	SWarp
+* Handle XML metada.
 *
-*	Author:		E.BERTIN (IAP)
+*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 *
-*	Contents:	XML logging.
+*	This file part of:	SWarp
 *
-*	Last modify:	13/07/2007
+*	Copyright:		(C) 2000-2011 Emmanuel Bertin -- IAP/CNRS/UPMC
 *
-*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-*/
+*	License:		GNU General Public License
+*
+*	SWarp is free software: you can redistribute it and/or modify
+*	it under the terms of the GNU General Public License as published by
+*	the Free Software Foundation, either version 3 of the License, or
+*	(at your option) any later version.
+*	SWarp is distributed in the hope that it will be useful,
+*	but WITHOUT ANY WARRANTY; without even the implied warranty of
+*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+*	GNU General Public License for more details.
+*	You should have received a copy of the GNU General Public License
+*	along with SWarp. If not, see <http://www.gnu.org/licenses/>.
+*
+*	Last modified:		07/06/2011
+*
+*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
 
 #ifdef HAVE_CONFIG_H
 #include	"config.h"
@@ -85,7 +97,7 @@ INPUT	-.
 OUTPUT	RETURN_OK if everything went fine, RETURN_ERROR otherwise.
 NOTES	Global preferences are used.
 AUTHOR	E. Bertin (IAP)
-VERSION	09/08/2006
+VERSION	11/04/2011
  ***/
 int	update_xml(fieldstruct *field, fieldstruct *wfield)
   {
@@ -106,7 +118,8 @@ int	update_xml(fieldstruct *field, fieldstruct *wfield)
   x->exptime = field->exptime;
   x->backmean = field->backmean;
   x->backsig = field->backsig;
-  x->sigfac = field->sigfac;
+  x->sigfac = wfield? wfield->sigfac : 1.0;
+  x->weight_thresh = wfield? wfield->weight_thresh : 0.0;
   x->gain = field->gain;
   x->saturation = field->saturation;
   x->fscale = field->fscale;
@@ -114,6 +127,7 @@ int	update_xml(fieldstruct *field, fieldstruct *wfield)
   x->pixscale = field->wcs->pixscale*DEG/ARCSEC;
   x->equinox = field->wcs->equinox;
   x->epoch = field->wcs->epoch;
+  x->obsdate = field->wcs->obsdate;
   x->naxis = field->wcs->naxis;
   x->celsys = (int)(field->wcs->celsysconvflag? field->wcs->celsys : -1);
   x->headflag = field->headflag;
@@ -235,7 +249,7 @@ INPUT	Pointer to the output file (or stream),
 OUTPUT	RETURN_OK if everything went fine, RETURN_ERROR otherwise.
 NOTES	-.
 AUTHOR	E. Bertin (IAP)
-VERSION	13/07/2007
+VERSION	07/06/2011
  ***/
 int	write_xml_meta(FILE *file, char *error)
   {
@@ -295,7 +309,7 @@ int	write_xml_meta(FILE *file, char *error)
 	" ucd=\"time.event.end;meta.software\" value=\"%s\"/>\n",
 	prefs.stime_end);
   fprintf(file, "  <PARAM name=\"Duration\" datatype=\"float\""
-	" ucd=\"time.event;meta.software\" value=\"%.0f\" unit=\"s\"/>\n",
+	" ucd=\"time.event;meta.software\" value=\"%.2f\" unit=\"s\"/>\n",
 	prefs.time_diff);
 
   fprintf(file, "  <PARAM name=\"User\" datatype=\"char\" arraysize=\"*\""
@@ -391,7 +405,9 @@ int	write_xml_meta(FILE *file, char *error)
 	" ucd=\"obs.param\" unit=\"pix\"/>\n");
   fprintf(file, "   <FIELD name=\"Back_Default\" datatype=\"float\""
 	" ucd=\"obs.param\" unit=\"adu\"/>\n");
-  fprintf(file, "   <FIELD name=\"Weight_Type\" datatype=\"char\""
+  fprintf(file, "   <FIELD name=\"Weight_Type\" datatype=\"boolean\""
+	" arraysize=\"*\" ucd=\"stat.weight;meta.code\"/>\n");
+  fprintf(file, "   <FIELD name=\"Rescale_Weights\" datatype=\"char\""
 	" arraysize=\"*\" ucd=\"stat.weight;meta.code\"/>\n");
   fprintf(file, "   <FIELD name=\"Weight_Thresh\" datatype=\"float\""
 	" ucd=\"instr.sensitivity;obs.param\" unit=\"adu\"/>\n");
@@ -414,6 +430,8 @@ int	write_xml_meta(FILE *file, char *error)
 	naxis, nxml? (xmlstack[0].celsys >=0? "deg":"pix") : "deg");
   fprintf(file, "   <FIELD name=\"Pixel_Scale\" datatype=\"float\""
 	" ucd=\"instr.pixel;obs.image;stat.mean\" unit=\"arcsec\"/>\n");
+  fprintf(file, "   <FIELD name=\"ObsDate\" datatype=\"double\""
+	" ucd=\"time.start;obs\" unit=\"yr\"/>\n");
   fprintf(file, "   <FIELD name=\"Equinox\" datatype=\"double\""
 	" ucd=\"time.equinox;obs\" unit=\"yr\"/>\n");
   fprintf(file, "   <FIELD name=\"Epoch\" datatype=\"double\""
@@ -427,10 +445,10 @@ int	write_xml_meta(FILE *file, char *error)
     f = x->fieldno;
     fprintf(file, "    <TR>\n"
 	"     <TD>%d</TD><TD>%s</TD><TD>%s</TD><TD>%c</TD><TD>%s</TD>\n"
-	"     <TD>%d</TD><TD>%s</TD><TD>%s</TD><TD>%.0f</TD>\n"
+	"     <TD>%d</TD><TD>%s</TD><TD>%s</TD><TD>%.2f</TD>\n"
 	"     <TD>%g</TD><TD>%g</TD><TD>%c</TD><TD>%s</TD>"
 	"<TD>%d</TD><TD>%d</TD><TD>%g</TD>\n"
-	"     <TD>%s</TD><TD>%g</TD><TD>%g</TD><TD>%c</TD>\n"
+	"     <TD>%s</TD><TD>%c</TD><TD>%g</TD><TD>%g</TD><TD>%c</TD>\n"
 	"     <TD>%g</TD><TD>%g</TD><TD>%g</TD><TD>%g</TD>\n"
 	"     <TD>%g</TD><TD>%.10g",
 	n,
@@ -453,7 +471,8 @@ int	write_xml_meta(FILE *file, char *error)
         prefs.back_default[f],
     	key[findkeys("WEIGHT_TYPE", keylist,
 		FIND_STRICT)].keylist[prefs.weight_type[f]],
-	prefs.weight_thresh[f],
+        prefs.wscale_flag[f]? 'T' : 'F',
+	x->weight_thresh,
 	x->sigfac,
         prefs.interp_flag[f]? 'T' : 'F',
 	x->gain,
@@ -485,8 +504,9 @@ int	write_xml_meta(FILE *file, char *error)
     for (d=1; d<naxis; d++)
       fprintf(file, " %.10g", x->centerpos[d]);
     fprintf(file,
-	"</TD><TD>%g</TD><TD>%g</TD><TD>%g</TD><TD>%s</TD>\n    </TR>\n",
-	x->pixscale, x->equinox, x->epoch, sysname);
+	"</TD><TD>%g</TD><TD>%.10g</TD><TD>%.10g</TD><TD>%.10g</TD>\n"
+	"     <TD>%s</TD>\n    </TR>\n",
+	x->pixscale, x->obsdate, x->equinox, x->epoch, sysname);
     }
   fprintf(file, "   </TABLEDATA></DATA>\n");
   fprintf(file, "  </TABLE>\n");
